"""App Tasks"""

# Standard Library
import json
import logging
from datetime import datetime, timedelta

# Third Party
import requests
from celery import shared_task
from invoices.models import Invoice

# Django
from django.utils import timezone

# Alliance Auth
from allianceauth.services.tasks import QueueOnce

# George Forge
from georgeforge.models import Order

from . import app_settings

logger = logging.getLogger(__name__)

# Create your tasks here
if app_settings.webhook_available():
    # Third Party
    from discord import Color, Embed

if app_settings.discord_bot_active():
    # Third Party
    from aadiscordbot.cogs.utils.exceptions import NotAuthenticated
    from aadiscordbot.tasks import send_message
    from aadiscordbot.utils.auth import get_discord_user_id


# Shamelessly yoinked from aa-securegroups/tasks.py
def send_discord_dm(user, title, message, color):
    if app_settings.discord_bot_active():
        try:
            e = Embed(title=title, description=message, color=color)
            try:
                send_message(user_id=get_discord_user_id(user), embed=e)
                logger.info(f"sent discord ping to {user} - {message}")
            except NotAuthenticated:
                logger.warning(f"Unable to ping {user} - {message}")

        except Exception as e:
            logger.error(e, exc_info=1)
            pass


def send_statusupdate_dm(order):
    if app_settings.discord_bot_active():
        match order.status:
            case Order.OrderStatus.PENDING:
                c = Color.blue()
            case Order.OrderStatus.AWAITING_DEPOSIT:
                c = Color.purple()
            case Order.OrderStatus.DEPOSIT_RECIEVED:
                c = Color.blue()
            case Order.OrderStatus.BUILDING_PARTS:
                c = Color.orange()
            case Order.OrderStatus.BUILDING_HULL:
                c = Color.orange()
            case Order.OrderStatus.AWAITING_FINAL_PAYMENT:
                c = Color.purple()
            case Order.OrderStatus.DELIVERED:
                c = Color.green()
            case Order.OrderStatus.REJECTED:
                c = Color.red()

        e = Embed(
            title=f"Order #{order.pk} Status: {order.get_status_display()}", color=c
        )
        e.add_field(name="Item", value=order.eve_type.name, inline=True)
        e.add_field(name="Quantity", value=str(order.quantity), inline=True)
        e.add_field(name="Price per Unit", value=f"{order.price:,.2f} ISK", inline=True)
        e.add_field(name="Total Cost", value=f"{order.totalcost:,.2f} ISK", inline=True)
        e.add_field(name="Deposit", value=f"{order.deposit:,.2f} ISK", inline=True)
        e.add_field(
            name="Delivery System", value=order.deliverysystem.name, inline=True
        )
        if order.estimated_delivery_date:
            e.add_field(
                name="Estimated Delivery",
                value=order.estimated_delivery_date,
                inline=True,
            )
        if order.description:
            e.add_field(name="Description", value=order.description, inline=False)
        if order.notes:
            e.add_field(name="Notes", value=order.notes, inline=False)
        if (
            order.status == Order.OrderStatus.PENDING
            and order.deposit > 0
            and app_settings.GEORGEFORGE_ORDER_DEPOSIT_INSTRUCTIONS
        ):
            e.add_field(
                name="Deposit Instructions",
                value=app_settings.GEORGEFORGE_ORDER_DEPOSIT_INSTRUCTIONS,
                inline=False,
            )

        try:
            send_message(user_id=get_discord_user_id(order.user), embed=e)
            logger.info(
                f"sent discord ping to {order.user} - order #{order.pk} status updated to {order.get_status_display()}"
            )
        except NotAuthenticated:
            logger.warning(
                f"Unable to ping {order.user} - order #{order.pk} status updated"
            )


def send_deliverydateupdate_dm(order):
    if app_settings.discord_bot_active():
        e = Embed(title=f"Order #{order.pk} Delivery Date Updated", color=Color.blue())

        if order.estimated_delivery_date:
            e.add_field(
                name="New Estimated Delivery",
                value=order.estimated_delivery_date,
                inline=False,
            )
        else:
            e.add_field(name="Estimated Delivery", value="Cleared", inline=False)

        e.add_field(name="Item", value=order.eve_type.name, inline=True)
        e.add_field(name="Quantity", value=str(order.quantity), inline=True)
        e.add_field(name="Price per Unit", value=f"{order.price:,.2f} ISK", inline=True)
        e.add_field(name="Total Cost", value=f"{order.totalcost:,.2f} ISK", inline=True)
        e.add_field(name="Deposit", value=f"{order.deposit:,.2f} ISK", inline=True)
        e.add_field(
            name="Delivery System", value=order.deliverysystem.name, inline=True
        )
        e.add_field(name="Status", value=order.get_status_display(), inline=True)
        if order.description:
            e.add_field(name="Description", value=order.description, inline=False)
        if order.notes:
            e.add_field(name="Notes", value=order.notes, inline=False)

        try:
            send_message(user_id=get_discord_user_id(order.user), embed=e)
            logger.info(
                f"sent discord ping to {order.user} - order #{order.pk} delivery date updated"
            )
        except NotAuthenticated:
            logger.warning(
                f"Unable to ping {order.user} - order #{order.pk} delivery date updated"
            )


@shared_task
def send_update_to_webhook(content=None, embed=None):
    web_hook = app_settings.GEORGEFORGE_ADMIN_WEBHOOK
    if web_hook is not None:
        custom_headers = {"Content-Type": "application/json"}
        payload = {}
        if embed:
            payload["embeds"] = [embed]
        if content:
            payload["content"] = content
        elif not embed:
            payload["content"] = "New order update"
        r = requests.post(
            web_hook,
            headers=custom_headers,
            data=json.dumps(payload),
        )
        logger.debug(f"Got status code {r.status_code} after sending ping")
        try:
            r.raise_for_status()
        except Exception as e:
            logger.error(e, exc_info=1)


@shared_task
def send_order_webhook(order_pk, updated=False, update_type=0):
    if not app_settings.webhook_available():
        return

    order = Order.objects.get(pk=order_pk)
    if not updated:
        embed = Embed(
            title=f"New Ship Order: {order.quantity} x {order.eve_type.name}",
            color=Color.blue(),
        )
        embed.add_field(
            name="Purchaser",
            value=order.user.profile.main_character.character_name,
            inline=True,
        )
        embed.add_field(name="Quantity", value=str(order.quantity), inline=True)
        embed.add_field(
            name="Price per Unit",
            value=f"{order.price:,.2f} ISK",
            inline=True,
        )
        embed.add_field(
            name="Total Cost",
            value=f"{order.totalcost:,.2f} ISK",
            inline=True,
        )
        embed.add_field(name="Deposit", value=f"{order.deposit:,.2f} ISK", inline=True)
        embed.add_field(
            name="Delivery System", value=order.deliverysystem.name, inline=True
        )
        embed.add_field(name="Status", value=order.get_status_display(), inline=True)
        if order.description:
            embed.add_field(name="Description", value=order.description, inline=False)
        if order.notes:
            embed.add_field(name="Notes", value=order.notes, inline=False)
    else:
        embed = Embed(title=f"Order #{order.id} updated!", color=Color.purple())
        embed.add_field(
            name="New status", value=f"```{order.get_status_display()}```", inline=True
        )
        embed.add_field(
            name="Order Date (M/D/Y)",
            value=f"```{str(datetime.now().strftime('%m/%d/%Y'))}```",  # i hate this as much as you do, i know
        )
        match update_type:
            case 0:  # DEPOSIT_PAID
                embed.add_field(
                    name="Deposit paid!",
                    value=f"```Invoice GF-DEP-{order.id} marked as paid.```",
                    inline=False,
                )
                embed.add_field(
                    name="Quantity", value=f"```{order.quantity}```", inline=True
                )
                embed.add_field(
                    name="Item", value=f"```{order.eve_type.name}```", inline=True
                )
                embed.add_field(
                    name="Type",
                    value=f"```{order.eve_type.eve_group.name}```",
                    inline=True,
                )
                embed.add_field(
                    name="Client",
                    value=f"```{order.user.profile.main_character.character_name}```",
                    inline=True,
                )
                embed.add_field(
                    name="Paid", value=f"```{order.paid:,.2f}```", inline=True
                )
                embed.add_field(
                    name="Order Total",
                    value=f"```{order.totalcost:,.2f}```",
                    inline=True,
                )
                embed.add_field(name="Notes", value=f"```{order.notes}```", inline=True)
            case 1:  # ADMIN_INVOICE
                embed.add_field(
                    name="Invoice missing - assuming this is an internal order!",
                    value=f"```Invoice GF-DEP-{order.id} missing.```",
                    inline=False,
                )
                embed.add_field(
                    name="Order details",
                    value=f"```{order.quantity} x {order.eve_type.name} for {order.user.profile.main_character.character_name}```",
                    inline=False,
                )

    content = None
    role_id = app_settings.GEORGEFORGE_ADMIN_WEBHOOK_ROLE_ID
    if role_id:
        content = f"<@&{role_id}>"

    send_update_to_webhook.delay(content=content, embed=embed.to_dict())


@shared_task(bind=True, base=QueueOnce)
def check_invoice_status(self):
    logger.info("Checking for complete Invoices")
    for order in Order.objects.filter(status=Order.OrderStatus.AWAITING_DEPOSIT).all():
        ref = f"GF-DEP-{str(order.id)}"
        try:
            inv = Invoice.objects.filter(invoice_ref=ref).get()
        except Invoice.DoesNotExist:
            order.status = Order.OrderStatus.DEPOSIT_RECIEVED
            order.save()
            send_order_webhook(order.id, True, 1)
            continue
        if inv.paid:
            order.paid += inv.amount
            order.status = Order.OrderStatus.DEPOSIT_RECIEVED
            order.save()
            send_order_webhook(order.id, True)


def send_order_invoice(order):
    if order.deposit != 0 and order.deposit > order.paid:
        isk = order.deposit - order.paid
        due = timezone.now() + timedelta(days=app_settings.GEORGEFORGE_DEPOSIT_DUE)
        character_id = order.user.profile.main_character.id
        inv = Order.generate_invoice(character_id, order.id, isk, due)
        if inv.amount < 1:
            logger.error(
                f"{order.deposit} - {order.paid} = {order.deposit - order.paid} or {isk}"
            )
            logger.error(print(inv))
            return 0
        else:
            inv.save()
            Order.ping_invoice(inv)



"""App Settings"""

# Django
from django.conf import settings


def discord_bot_active():
    return "aadiscordbot" in settings.INSTALLED_APPS


def webhook_available():
    try:
        # Third Party
        import discord

        return discord is not None
    except ImportError:
        return False


# Name of this app as shown in the Auth sidebar, page titles
GEORGEFORGE_APP_NAME = getattr(settings, "GEORGEFORGE_APP_NAME", "George Forge")

GEORGEFORGE_CATEGORIES = getattr(
    settings, "GEORGEFORGE_CATEGORIES", [4, 6, 7, 8, 18, 20, 63, 66]
)

GEORGEFORGE_ADMIN_WEBHOOK = getattr(settings, "GEORGEFORGE_ADMIN_WEBHOOK", None)

GEORGEFORGE_ADMIN_WEBHOOK_ROLE_ID = getattr(
    settings, "GEORGEFORGE_ADMIN_WEBHOOK_ROLE_ID", None
)

GEORGEFORGE_ORDER_PENDING_DEFAULT = getattr(
    settings, "GEORGEFORGE_ORDER_PENDING_DEFAULT", True
)

GEORGEFORGE_DEPOSIT_DUE = getattr(settings, "GEORGEFORGE_DEPOSIT_DUE", 7)

GEORGEFORGE_ORDER_DEPOSIT_INSTRUCTIONS = getattr(
    settings, "GEORGEFORGE_ORDER_DEPOSIT_INSTRUCTIONS", None
)



"""Template tags for georgeforge"""

# Django
from django import template
from django.utils.translation import gettext_lazy as _

# Alliance Auth (External Libs)
# eve_sde
from eve_sde.models import ItemType

# George Forge
from georgeforge import app_settings

register = template.Library()


@register.filter
def has_discord_linked(user):
    """Check if user has discord linked using get_discord_user_id"""
    if not app_settings.discord_bot_active():
        return True

    try:
        # Third Party
        from aadiscordbot.cogs.utils.exceptions import NotAuthenticated
        from aadiscordbot.utils.auth import get_discord_user_id

        get_discord_user_id(user)
        return True
    except NotAuthenticated:
        return False
    except ImportError:
        return True


@register.filter
def evetype_icon(eve_type, size=32):
    """Take an eve_type object and return an image HTML element of it's icon WITH an item level overlay"""
    base_icon_url = f"https://images.evetech.net/types/{eve_type.id}/icon?size=64"
    pip = ""
    tl = ItemType.objects.get(id=eve_type.id).meta_group_id_raw
    if tl is None or tl == 1:
        pip = ""
    else:
        pip = f'<img class="pip" src="/static/georgeforge/img/{int(tl)}.png" />'

    return f'{pip}<img height="{size}" width="{size}" src="{base_icon_url}" alt="{_("Item Icon")}" />'



# Generated by Django 4.2.29 on 2026-03-26 01:33

# Django
import django.core.validators
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    replaces = [
        (
            "georgeforge",
            "0001_squashed_0020_alter_order_deposit_alter_order_paid_and_more",
        ),
        ("georgeforge", "0021_alter_order_deliverysystem"),
        ("georgeforge", "0022_add_order_cart_session_id"),
        ("georgeforge", "0023_remove_cart_session_id_unique"),
        ("georgeforge", "0024_add_order_estimated_delivery_date"),
        ("georgeforge", "0025_alter_estimated_delivery_date_to_string"),
        ("georgeforge", "0026_remove_order_on_behalf_of"),
        ("georgeforge", "0027_alter_order_estimated_delivery_date"),
        ("georgeforge", "0028_alter_order_estimated_delivery_date"),
        ("georgeforge", "0029_alter_order_status"),
        (
            "georgeforge",
            "0030_alter_deliverysystem_system_alter_forsale_eve_type_and_more",
        ),
    ]

    dependencies = [
        ("eve_sde", "0018_blueprintactivity_blueprintactivityproduct_and_more"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="General",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "permissions": (
                    ("place_order", "Can place an order"),
                    ("manage_store", "Can manage the store"),
                ),
                "managed": False,
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="Order",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "price",
                    models.DecimalField(
                        decimal_places=2,
                        help_text="Cost per unit",
                        max_digits=15,
                        verbose_name="Price per unit",
                    ),
                ),
                (
                    "totalcost",
                    models.DecimalField(
                        decimal_places=2,
                        help_text="Total Order cost",
                        max_digits=25,
                        verbose_name="Total Order cost",
                    ),
                ),
                (
                    "deposit",
                    models.DecimalField(
                        decimal_places=2,
                        help_text="Deposit required per unit",
                        max_digits=25,
                        verbose_name="Deposit required per unit",
                    ),
                ),
                (
                    "paid",
                    models.DecimalField(
                        decimal_places=2,
                        default=0,
                        help_text="Amount paid",
                        max_digits=25,
                        verbose_name="Amount Paid",
                    ),
                ),
                (
                    "quantity",
                    models.PositiveIntegerField(
                        default=1,
                        validators=[django.core.validators.MinValueValidator(1)],
                        verbose_name="Quantity",
                    ),
                ),
                (
                    "notes",
                    models.TextField(blank=True, max_length=4096, verbose_name="Notes"),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True, max_length=4096, verbose_name="Description"
                    ),
                ),
                (
                    "status",
                    models.IntegerField(
                        choices=[
                            (10, "Pending"),
                            (20, "Awaiting Deposit"),
                            (25, "Deposit Recieved"),
                            (30, "Building Parts"),
                            (35, "Building Hull"),
                            (40, "Contract Up"),
                            (50, "Delivered"),
                            (60, "Rejected"),
                        ],
                        verbose_name="Status",
                    ),
                ),
                (
                    "cart_session_id",
                    models.CharField(
                        db_index=True, max_length=64, verbose_name="Cart Session ID"
                    ),
                ),
                (
                    "estimated_delivery_date",
                    models.CharField(
                        blank=True,
                        help_text="Estimated date when the order will be delivered",
                        max_length=50,
                        verbose_name="Estimated Delivery Date",
                    ),
                ),
                (
                    "deliverysystem",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="eve_sde.solarsystem",
                        verbose_name="Delivery System",
                    ),
                ),
                (
                    "eve_type",
                    models.ForeignKey(
                        limit_choices_to={"published": 1},
                        on_delete=django.db.models.deletion.CASCADE,
                        to="eve_sde.itemtype",
                        verbose_name="EVE Type",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.RESTRICT,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Purchaser",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="ForSale",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True, max_length=4096, verbose_name="Description"
                    ),
                ),
                (
                    "price",
                    models.DecimalField(
                        decimal_places=2,
                        help_text="Cost per unit",
                        max_digits=15,
                        validators=[django.core.validators.MinValueValidator(1)],
                        verbose_name="Price",
                    ),
                ),
                (
                    "deposit",
                    models.DecimalField(
                        decimal_places=2,
                        default=0,
                        help_text="Deposit per unit",
                        max_digits=15,
                        verbose_name="Deposit",
                    ),
                ),
                (
                    "eve_type",
                    models.ForeignKey(
                        limit_choices_to={"published": 1},
                        on_delete=django.db.models.deletion.CASCADE,
                        to="eve_sde.itemtype",
                        verbose_name="EVE Type",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="DeliverySystem",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "friendly_name",
                    models.TextField(
                        blank=True,
                        default=None,
                        max_length=32,
                        null=True,
                        verbose_name="Friendly Name",
                    ),
                ),
                ("enabled", models.BooleanField(default=True)),
                (
                    "system",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="eve_sde.solarsystem",
                        verbose_name="Solar System",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
    ]



# Generated by Django 4.2.29 on 2026-03-26 01:30

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("eve_sde", "0018_blueprintactivity_blueprintactivityproduct_and_more"),
        ("georgeforge", "0029_alter_order_status"),
    ]

    operations = [
        migrations.AlterField(
            model_name="deliverysystem",
            name="system",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="eve_sde.solarsystem",
                verbose_name="Solar System",
            ),
        ),
        migrations.AlterField(
            model_name="forsale",
            name="eve_type",
            field=models.ForeignKey(
                limit_choices_to={"published": 1},
                on_delete=django.db.models.deletion.CASCADE,
                to="eve_sde.itemtype",
                verbose_name="EVE Type",
            ),
        ),
        migrations.AlterField(
            model_name="order",
            name="deliverysystem",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="eve_sde.solarsystem",
                verbose_name="Delivery System",
            ),
        ),
        migrations.AlterField(
            model_name="order",
            name="eve_type",
            field=models.ForeignKey(
                limit_choices_to={"published": 1},
                on_delete=django.db.models.deletion.CASCADE,
                to="eve_sde.itemtype",
                verbose_name="EVE Type",
            ),
        ),
    ]



# Generated by Django 4.2.20 on 2025-07-25 00:11

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "georgeforge",
            "0001_squashed_0020_alter_order_deposit_alter_order_paid_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="order",
            name="deliverysystem",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="eveuniverse.evesolarsystem",
                verbose_name="Delivery System",
            ),
        ),
    ]



# Generated migration

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("georgeforge", "0024_add_order_estimated_delivery_date"),
    ]

    operations = [
        migrations.AlterField(
            model_name="order",
            name="estimated_delivery_date",
            field=models.CharField(
                blank=True,
                help_text="Estimated date when the order will be delivered",
                max_length=50,
                null=True,
                verbose_name="Estimated Delivery Date",
            ),
        ),
    ]



"""Admin models"""

# Django
from django.contrib import admin

# George Forge
from georgeforge.models import DeliverySystem, ForSale, Order


class ManageStoreAdmin(admin.ModelAdmin):
    def _has_perm(self, request) -> bool:
        return request.user.has_perm("georgeforge.manage_store")

    def has_view_permission(self, request, obj=None):
        return self._has_perm(request)

    def has_add_permission(self, request):
        return self._has_perm(request)

    def has_change_permission(self, request, obj=None):
        return self._has_perm(request)

    def has_delete_permission(self, request, obj=None):
        return self._has_perm(request)


@admin.register(ForSale)
class ForSaleAdmin(ManageStoreAdmin):
    """ """

    list_display = ["eve_type", "description", "deposit", "price"]
    # autocomplete_fields = ["eve_type"]


@admin.register(DeliverySystem)
class DeliverySystemAdmin(ManageStoreAdmin):
    """ """

    list_display = ["system", "enabled", "friendly_name"]
    # autocomplete_fields = ["system"]


@admin.register(Order)
class OrderAdmin(ManageStoreAdmin):
    """ """

    list_display = [
        "user",
        "status",
        "eve_type",
        "price",
        "description",
        "notes",
        "estimated_delivery_date",
        "cart_session_id",
    ]
    # autocomplete_fields = ["eve_type"]



# Generated by Django 4.2.27 on 2026-01-24 20:14

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("georgeforge", "0028_alter_order_estimated_delivery_date"),
    ]

    operations = [
        migrations.AlterField(
            model_name="order",
            name="status",
            field=models.IntegerField(
                choices=[
                    (10, "Pending"),
                    (20, "Awaiting Deposit"),
                    (25, "Deposit Recieved"),
                    (30, "Building Parts"),
                    (35, "Building Hull"),
                    (40, "Contract Up"),
                    (50, "Delivered"),
                    (60, "Rejected"),
                ],
                verbose_name="Status",
            ),
        ),
    ]



# Generated manually

# Standard Library
import uuid

# Django
from django.db import migrations, models


def backfill_cart_session_ids(apps, schema_editor):
    Order = apps.get_model("georgeforge", "Order")

    for order in Order.objects.filter(cart_session_id__isnull=True):
        order.cart_session_id = str(uuid.uuid4())
        order.save(update_fields=["cart_session_id"])


def reverse_backfill(apps, schema_editor):
    pass


class Migration(migrations.Migration):
    dependencies = [
        ("georgeforge", "0022_add_order_cart_session_id"),
    ]

    operations = [
        migrations.RunPython(backfill_cart_session_ids, reverse_backfill),
        migrations.AlterField(
            model_name="order",
            name="cart_session_id",
            field=models.CharField(
                blank=False,
                db_index=True,
                max_length=64,
                null=False,
                verbose_name="Cart Session ID",
            ),
        ),
    ]
